# encoding: UTF-8
require 'thor'
require_relative 'helper'
require_relative 'fpa_cli'
require_relative 'pert_cli'
require_relative 'trace_cli'
include Minireq::Cli
include Minireq::Core::Reader # TODO move to helper?

module Minireq
  module Cli

    class Cli < Thor
      include Thor::Actions

      def self.source_root
        Minireq.root
      end

      map %w[--version -v] => :__print_version

      desc "--version, -v", "Print the version"
      def __print_version
        puts "Minireq #{Minireq::VERSION}"
      end

      desc "new TARGET", "Create a new project"
      def new(target)
        Dir.mkdir(target)
        Dir.chdir(target) do
          create_project

          config = {}
          config.merge!({constant_name: target})
          template("lib/minireq/templates/README.md.tt", "#{target}/README.md", config)
          copy_file("lib/minireq/templates/creq.thor", "#{target}/creq.thor")

          say "Initializing git repo in #{target}"
          `git init`
          `git add .`
        end
      end

      desc "promo", "Copy 'promo' project content"
      def promo
        promo_dir = File.join Minireq.root, 'lib/minireq/promo'
        directory promo_dir, Dir.pwd
      end

      desc "init", "Initialize empty repository if not exists"
      def init
        create_project #TODO refactor it to init_project
        puts "Minireq project struture created successfully!"
      end

      desc "req id [TITLE]", "Create requirement file id.md"
      method_option :template, aliases: "-t", type: :string,
        desc: "Template file for body section"
      # --option=name:string age:integer
      method_option :attributes, aliases: "-a", type: :hash,
        :default => {}, required: false, desc: "Attributes"
      def req(id, title = '')
        errmsg = "File #{REQ}/#{id}.md already exists. Operation aborted."
        if File.exist?("#{REQ}/#{id}.md")
          puts errmsg
          return
        end

        params = {}
        params.merge!(title: title) unless title.empty?
        params.merge!(options[:attributes]) if options[:attributes]
        params.merge!({body: File.read(options[:template])}) if options[:template]
        # replace @@id
        params[:body].gsub!('@@id', id) if params[:body]
        params[:body].gsub!('@@title', title) if params[:body] && !title.empty?
        create_requirement_source(id, params)
        puts "#{REQ}/#{id}.md created."
      end

      desc "check", "Check requirement repository"
      def check
        doc = Minireq::Core::Document.new
        file_repo = inreq { read_requirement_files }
        errors = doc.check_errors(file_repo)
        unless errors.empty?
          errmsg = ""
          if errors[:nonuniq_ids]
            errmsg << "Non-unique requirement ids:\n"
            errmsg << errors[:nonuniq_ids].join("\n")
            errmsg << "\n"
          end
          if errors[:wrong_links]
            errmsg << "Wrong requirements links:\n"
            errmsg << errors[:wrong_links].join("\n")
            errmsg << "\n"
          end
          puts errmsg
          return
        end
        puts "Everything is fine!"
      end

      desc "doc [id]", "Create requirements document"
      method_option :output, aliases: "-o", type: :string, desc: "Output file"
      def doc(id = nil)
        if id
          if write_req(id)
            puts "'#{DOC}/#{id}.md' created."
          else
            # TODO show all possible requirement ids from repo
            puts "Requirement '#{id}' not found in repository!"
            return
          end
        else
          file_name = options[:output]
          file_name ||= 'requirements.md'
          write_doc(file_name)
          puts "'#{DOC}/#{file_name}' created."
        end
      end

      desc "trace SUBCOMMAND ...ARGS", "Trace"
      subcommand "trace", TraceCli

      desc "fpa SUBCOMMAND ...ARGS", "FPA estimation"
      subcommand "fpa", FpaCli

      desc "pert SUBCOMMAND ...ARGS", "FPA estimation"
      subcommand "pert", PertCli

    end
  end
end
