#encoding: UTF-8
require 'forwardable'
require_relative 'fpa'

module Minireq
  module Dsl

=begin
      FPA calculator
      Usage
      ```ruby
      include Minireq::Calc::FPA
      estimate do
        ILF 'Product',  det: 20, ret: 3
        ILF 'Customer', det: 20, ret: 2
        ILF 'Order',    det: 10, ret: 1

        EO 'CustomerView', det: 20, ftr: 1
        EI 'CustomerEdit', det: 20, ftr: 2
        EQ 'CustomerEdit', det: 20, ftr: 2

        characteristics ({
          data_communications:         0,
          distributed_data_processing: 0,
          performance:                 0,
          heavily_used_configuration:  0,
          transaction_rate:            0,
          online_data_entry:           0,
          enduser_efficiency:          3,
          online_update:               3,
          complex_processing:          3,
          reusability:                 0,
          installation_ease:           3,
          operational_ease:            3,
          multiple_sites:              0,
          facilitate_change:           3
        })
        puts calculate
      end
      ```
=end
      class FpaDSL
        extend Forwardable
        include Fpa

        # [Hash<Object>]
        attr_reader :items

        # @!macro [attach] def_delegators
        #   @!method $2
        #     Forwards to $1.
        #     @see Hash#$2
        def_delegator :@items, :[]
        def_delegator :@items, :size
        def_delegator :@items, :clear

        def initialize
          @items = {}
          @characteristics = {}
          @ufp = 0
        end

        def estimate(&block)
          instance_eval(&block) if block_given?
          calculate
        end

        def ILF(object, det:, ret:)
          hold(object, fun: :ILF, det: det, ret: ret)
          @ufp += ufp(type: :ILF, det: det, ret: ret)
        end

        def EIF(object, det:, ret:)
          hold(object, fun: :EIF, det: det, ret: ret)
          @ufp += ufp(type: :EIF, det: det, ret: ret)
        end

        def EI(object, det:, ftr:)
          hold(object, fun: :EI, det: det, ftr: ftr)
          @ufp += ufp(type: :EI, det: det, ftr: ftr)
        end

        def EO(object, det:, ftr:)
          hold(object, fun: :EO, det: det, ftr: ftr)
          @ufp += ufp(type: :EO, det: det, ftr: ftr)
        end

        def EQ(object, det:, ftr:)
          hold(object, fun: :EQ, det: det, ftr: ftr)
          @ufp += ufp(type: :EQ, det: det, ftr: ftr)
        end

        def characteristics(characteristics)
          @characteristics = characteristics
        end

        def calculate
          @vaf = vaf(@characteristics)
          {ufp: @ufp, vaf: @vaf, fp: @ufp * @vaf}
        end

        private

          def hold(obj, params)
            if @items[obj]
              # TODO rescue thi error
              raise MinireqError, "An attempt of adding a duplicate estimation '#{params[:fun]}' for '#{obj}'"
            end
            @items[obj] = params
          end
      end
  end
end
